@@ -8,14 +8,30 @@ Documentation: http://hxlstandard.org
 
 """
 
-__version__="1.27.1"
+__version__="1.28"
 """Module version number
 See https://www.python.org/dev/peps/pep-0396/
 
 """
 
 #
-# Continue with other imports
+# Ignore nuisance warnings from flask_caching (for now)
+#
+
+import warnings
+warnings.filterwarnings(
+    'ignore',
+    category=UserWarning,
+    module='flask_caching',
+)
+warnings.filterwarnings(
+    'ignore',
+    category=DeprecationWarning,
+    module='flask_caching',
+)
+
+#
+# Continue imports
 #
 import flask, flask_caching, json, os, requests_cache, werkzeug.datastructures
 from . import reverse_proxied
@@ -46,7 +62,7 @@ app.jinja_env.lstrip_blocks = True
 #
 # Set up logging before any other imports, to get correct format in server logs
 #
-import logging, logging.config
+import logging, logging.config, logging.handlers
 
 
 
@@ -70,6 +86,9 @@ logging.config.dictConfig(
                 'format': '[%(asctime)s] %(levelname)s in %(name)s: %(message)s',
             },
             # This is where we can implement JSON formatting later
+            'plain': {
+                'format': '%(message)s',
+            },
         },
         'handlers': {
             # default handler for everything but "hxl.REMOTE_ACCESS"
@@ -82,9 +101,9 @@ logging.config.dictConfig(
             },
             # special handler for "hxl.REMOTE_ACCESS"
             'remote_access': {
-                'class': 'logging.StreamHandler',
-                'stream': 'ext://flask.logging.wsgi_errors_stream',
-                'formatter': 'default',
+                'class': 'logging.handlers.WatchedFileHandler',
+                'filename': app.config.get('REMOTE_ACCESS_LOG_FILE', "/var/log/proxy/hxl.log"),
+                'formatter': 'plain',
                 'level': app.config.get('REMOTE_ACCESS_LOGGING_LEVEL', 'INFO'),
                 'filters': ['remote_access'],
             },
@@ -113,6 +132,24 @@ logging.config.dictConfig(
 logger = logging.getLogger(__name__)
 """ Python logger for this module """
 
+import structlog
+logger = structlog.wrap_logger(logging.getLogger('hxl.REMOTE_ACCESS'))
+
+structlog_processors = [
+    structlog.contextvars.merge_contextvars,
+    structlog.processors.add_log_level,
+    structlog.processors.StackInfoRenderer(),
+    structlog.dev.set_exc_info,
+    structlog.processors.TimeStamper(fmt="iso"),
+    structlog.processors.dict_tracebacks,
+]
+
+if app.config.get('ENV_TYPE', 'PROD') != 'PROD':
+    structlog_processors.append(structlog.dev.ConsoleRenderer())
+else:
+    structlog_processors.append(structlog.processors.JSONRenderer())
+
+structlog.configure(structlog_processors)
 
 #
 # Set up output cache
