@@ -13,19 +13,38 @@ from hxl.input import HXLIOException
 
 from hxl_proxy import admin, app, auth, cache, caching, dao, exceptions, filters, pcodes, preview, recipes, util, validate
 
-import datetime, flask, hxl, io, json, logging, requests, requests_cache, werkzeug, csv, urllib
+import datetime, flask, hxl, io, json, logging, requests, requests_cache, signal, werkzeug, csv, urllib
 
+from hxl.util import logup
 
 logger = logging.getLogger(__name__)
 """ Python logger for this module """
 
+from structlog import contextvars, get_logger, wrap_logger
+input_logger = wrap_logger(logging.getLogger('hxl.REMOTE_ACCESS'))
+
 SHEET_MAX_NO = 20
 
+
+########################################################################
+# Asynchronous signal handlers
+########################################################################
+
+def handle_alarm_signal(signum, frame):
+    """ Raise a TimeoutError when there's an alarm """
+    logging.warning("Request timed out")
+    logup('Request timed out', level='info')
+    raise TimeoutError()
+
+signal.signal(signal.SIGALRM, handle_alarm_signal)
+
+
 
 ########################################################################
 # Error handlers
 #
-# These functions handle exceptions that make it to the top level.
+# These functions handle exceptions that make it to the top level, as
+# well as a timeout.
 #
 # The HXL Proxy uses exceptions for special purposes like redirections
 # or authorisation as well as errors.
@@ -38,15 +57,21 @@ def handle_default_exception(e):
     @param e: the exception being handled
     """
 
+    key_error = {'error_type': type(e).__name__}
     # log a warning for the error/exception that we're handling
     if hasattr(e, 'message'):
+        key_error['message'] = e.message
         logger.warning('%s: %s', type(e).__name__, e.message)
     else:
         logger.warning(type(e).__name__)
+    logup('Error', key_error, "warning")
+
 
     if isinstance(e, requests.exceptions.HTTPError) or isinstance(e, hxl.input.HXLHTMLException):
         status = 404
         e = exceptions.RemoteDataException(e)
+    elif isinstance(e, TimeoutError): # more specific than the following
+        status = 408 # HTTP timeout
     elif isinstance(e, IOError) or isinstance(e, OSError):
         # probably tried to open an inappropriate URL
         status = 403
@@ -67,8 +92,7 @@ def handle_default_exception(e):
         return flask.render_template('error.html', e=e, category=type(e)), status
 
 # Register the general error handler UNLESS we're in debug mode
-if not app.config.get('DEBUG'):
-    app.register_error_handler(Exception, handle_default_exception)
+app.register_error_handler(Exception, handle_default_exception)
 
 
 def handle_redirect_exception(e):
@@ -137,24 +161,6 @@ def handle_password_required_exception(e):
 app.register_error_handler(werkzeug.exceptions.Unauthorized, handle_password_required_exception)
 
 
-def handle_ssl_certificate_error(e):
-    """ Error handler: SSL certificate error
-    Give the user an option to disable SSL certificate verification by redirecting to the /data/source page.
-    @param e: the exception being handled
-    """
-    if flask.g.output_format == "html":
-        flask.flash("SSL error. If you understand the risks, you can check \"Don't verify SSL certificates\" to continue.")
-        return flask.redirect(util.data_url_for('data_source', recipe=recipes.Recipe()), 302)
-    else:
-        response = flask.Response(util.make_json_error(e, 400), mimetype='application/json', status=400)
-        # add CORS header
-        response.headers['Access-Control-Allow-Origin'] = '*'
-        return response
-
-# register the SSL certificate verification handler
-app.register_error_handler(requests.exceptions.SSLError, handle_ssl_certificate_error)
-
-
 
 ########################################################################
 # Global pre-/post-controller functions
@@ -176,6 +182,14 @@ def before_request():
     # select the default output format (controllers may change it)
     flask.g.output_format='html'
 
+    # set the timeout for the request
+    try:
+        timeout = int(app.config.get('TIMEOUT', 30))
+    except ValueError:
+        timeout = 30
+    signal.alarm(timeout)
+    
+
 
 
 ########################################################################
@@ -206,25 +220,9 @@ def about():
 # /data GET controllers
 ########################################################################
 
-# has tests
-@app.route("/data/<recipe_id>/login")
-def data_login(recipe_id):
-    """ Flask controller: log in to work on a saved recipe
-    The user will end up here only if they tried to alter a saved
-    recipe. They will have to enter the recipe's password to
-    continue.
-    @param recipe_id: the hash for a saved recipe (or None if working from the command line)
-    """
-    flask.g.recipe_id = recipe_id # for error handling
-    recipe = recipes.Recipe(recipe_id)
-    destination = flask.request.args.get('from')
-    if not destination:
-        destination = util.data_url_for('data_edit', recipe)
-    return flask.render_template('data-login.html', recipe=recipe, destination=destination)
-
 # has tests
 @app.route("/data/source")
-@app.route("/data/<recipe_id>/source")
+@util.structlogged
 def data_source(recipe_id=None):
     """ Flask controller: choose a new source URL
     @param recipe_id: the hash for a saved recipe (or None if working from the command line)
@@ -235,7 +233,7 @@ def data_source(recipe_id=None):
 
 # has tests
 @app.route("/data/tagger")
-@app.route("/data/<recipe_id>/tagger")
+@util.structlogged
 def data_tagger(recipe_id=None):
     """ Flask controller: add HXL tags to an untagged dataset
     The template will render differently depending on whether the user has selected the
@@ -250,6 +248,7 @@ def data_tagger(recipe_id=None):
 
     # Workflow: if there's no source URL, redirect the user to /data/source
     if not recipe.url:
+        logup('No URL supplied for /data/tagger; redirecting to /data/source', level='info')
         logger.info("No URL supplied for /data/tagger; redirecting to /data/source")
         flask.flash('Please choose a data source first.')
         return flask.redirect(util.data_url_for('data_source', recipe), 303)
@@ -284,7 +283,7 @@ def data_tagger(recipe_id=None):
 
 # has tests
 @app.route("/data/edit")
-@app.route("/data/<recipe_id>/edit", methods=['GET', 'POST'])
+@util.structlogged
 def data_edit(recipe_id=None):
     """Flask controller: create or edit a filter pipeline.
     Output for this page is never cached, but input may be.
@@ -298,6 +297,7 @@ def data_edit(recipe_id=None):
     # Workflow: if there's no source URL, redirect the user to /data/source
     if not recipe.url:
         flask.flash('Please choose a data source first.')
+        logup('No URL supplied for /data/edit; redirecting to /data/source', level="info")
         logger.info("No URL supplied for /data/edit; redirecting to /data/source")
         return flask.redirect(util.data_url_for('data_source', recipe), 303)
 
@@ -349,7 +349,7 @@ def data_edit(recipe_id=None):
 
 # has tests
 @app.route("/data/save")
-@app.route("/data/<recipe_id>/save")
+@util.structlogged
 def data_save(recipe_id=None):
     """ Flask controller: create or update a saved dataset (with a short URL)
     The user will get redirected here automatically if they attempt to open a private
@@ -366,6 +366,7 @@ def data_save(recipe_id=None):
     # Workflow: if there's no source URL, redirect the user to /data/source
     if not recipe.url:
         flask.flash('Please choose a data source first.')
+        logup('No URL supplied for /data/save; redirecting to /data/source', level="info")
         logger.info("No URL supplied for /data/save; redirecting to /data/source")
         return flask.redirect(util.data_url_for('data_source', recipe), 303)
 
@@ -381,6 +382,7 @@ def data_save(recipe_id=None):
 @app.route("/data/validate.<format>")
 @app.route("/data/<recipe_id>/validate")
 @app.route("/data/<recipe_id>/validate.<format>")
+@util.structlogged
 def data_validate(recipe_id=None, format='html'):
     """ Flask controller: validate a HXL dataset and show the results
     Output options include a web-based HTML dashboard or JSON.
@@ -398,6 +400,7 @@ def data_validate(recipe_id=None, format='html'):
     # Workflow: if there's no source URL, redirect the user to /data/source
     if not recipe.url:
         flask.flash('Please choose a data source first.')
+        logup('No URL supplied for /data/validate; redirecting to /data/source', level="info")
         logger.info("No URL supplied for /data/validate; redirecting to /data/source")
         return flask.redirect(util.data_url_for('data_source', recipe), 303)
 
@@ -405,8 +408,10 @@ def data_validate(recipe_id=None, format='html'):
     schema_source = None
     if recipe.schema_url:
         schema_source = util.hxl_data(recipe.schema_url, util.make_input_options(recipe.args))
+        logup('Using HXL validation schema', {"schema": recipe.schema_url}, level="info")
         logger.info("Using HXL validation schema at %s", recipe.schema_url)
     else:
+        logup('No HXL validation schema specified; using default schema', level="info")
         logger.info("No HXL validation schema specified; using default schema")
 
     # Run the validation and get a JSON report from libhxl-python
@@ -434,6 +439,7 @@ def data_validate(recipe_id=None, format='html'):
 
         # if there's a detail_hash, show just that detail in the report
         if detail_hash:
+            logup('Showing validation-report detail', level="info")
             logger.info("Showing validation-report detail")
             template_name = 'validate-issue.html'
             for issue in error_report['issues']:
@@ -454,7 +460,7 @@ def data_validate(recipe_id=None, format='html'):
 
 # has tests
 @app.route("/data/advanced")
-@app.route("/data/<recipe_id>/advanced")
+@util.structlogged
 def show_advanced(recipe_id=None):
     """ Flask controller: developer page for entering a JSON recipe directly
     This page isn't linked from the HXML Proxy validation, but it's a convenient
@@ -468,6 +474,7 @@ def show_advanced(recipe_id=None):
 # no tests
 @app.route("/data/logs")
 @app.route("/data/<recipe_id>/logs")
+@util.structlogged
 def data_logs(recipe_id=None):
     """ Flask controller: show logs for a recipe
     """
@@ -488,6 +495,7 @@ def data_logs(recipe_id=None):
 @app.route("/data/<recipe_id>/download/<stub>.<format>")
 @app.route("/data/<recipe_id>") # must come last, or it will steal earlier patterns
 @cache.cached(key_prefix=util.make_cache_key, unless=util.skip_cache_p)
+@util.structlogged
 def data_view(recipe_id=None, format="html", stub=None, flavour=None):
     """ Flask controller: render a transformed dataset
     This is the controller that requests will hit most of the time.
@@ -610,6 +618,7 @@ def data_view(recipe_id=None, format="html", stub=None, flavour=None):
 
 # needs tests
 @app.route("/actions/login", methods=['POST'])
+@util.structlogged
 def do_data_login():
     """ Flask controller: log the user in for a specific dataset.
     Note that this is NOT a user login; it's a dataset login. That
@@ -636,6 +645,7 @@ def do_data_login():
 
 # needs tests
 @app.route("/actions/save-recipe", methods=['POST'])
+@util.structlogged
 def do_data_save():
     """ Flask controller: create or update a saved recipe
     The saved recipe has all of its parameters in the database, and is
@@ -740,6 +750,7 @@ def do_data_save():
 
 # has tests
 @app.route("/actions/validate", methods=['POST'])
+@util.structlogged
 def do_data_validate():
     """ Flask controller: validate an uploaded file against an uploaded HXL schema
     This controller was created for HDX Data Check, which is the only known user.
@@ -772,6 +783,7 @@ def do_data_validate():
         try:
             sheet_index = int(sheet_index)
         except:
+            logup('Bad sheet index', {"sheet": flask.request.form.get('sheet')}, level='warning')
             logger.warning("Bad sheet index: %s", flask.request.form.get('sheet'))
             sheet_index = None
     selector = flask.request.form.get('selector', None)
@@ -788,6 +800,7 @@ def do_data_validate():
         try:
             schema_sheet_index = int(schema_sheet_index)
         except:
+            logup('Bad schema_sheet index', {"sheet": flask.request.form.get('schema_sheet')}, level='warning')
             logger.warning("Bad schema_sheet index: %s", flask.request.form.get('schema_sheet'))
             schema_sheet_index = None
 
@@ -823,6 +836,7 @@ def do_data_validate():
 # NOTE: This is an experiment that's probably not used anywhere right now
 # We may choose to remove it
 @app.route('/actions/json-spec', methods=['POST'])
+@util.structlogged
 def do_json_recipe():
     """ POST handler to execute a JSON recipe
     This POST endpoint allows the user to upload a JSON HXL recipe
@@ -889,6 +903,7 @@ def do_json_recipe():
 ########################################################################
 
 @app.route('/login')
+@util.structlogged
 def hid_login():
     """ Flask controller: display login page for Humanitarian.ID
     This is distinct from the /data/login page, which accepts a password
@@ -909,6 +924,7 @@ def hid_login():
 
 
 @app.route('/logout')
+@util.structlogged
 def hid_logout():
     """ Flask controller: kill the user's login session with Humanitarian.ID
 
@@ -924,6 +940,7 @@ def hid_logout():
 
 # not currently in use (until we reactivate H.ID support)
 @app.route('/settings/user')
+@util.structlogged
 def user_settings():
     """ Flask controller: show the user's settings from Humanitarian.ID
     """
@@ -937,6 +954,7 @@ def user_settings():
 
 
 @app.route('/oauth/authorized2/1')
+@util.structlogged
 def do_hid_authorisation():
     """Flask controller: accept an OAuth2 token after successful login via Humanitarian.ID
 
@@ -967,108 +985,6 @@ def do_hid_authorisation():
     return flask.redirect(redirect_path, 303)
 
 
-
-########################################################################
-# /admin controllers
-########################################################################
-
-
-# needs tests
-@app.route("/admin/login")
-def admin_login():
-    """ Log in to use admin functions """
-    return flask.render_template('admin-login.html')
-
-
-# needs tests
-@app.route("/admin/recipes/<recipe_id>/")
-def admin_recipe_view(recipe_id):
-    """ View a specific recipe """
-    admin.admin_auth()
-    recipe = recipes.Recipe(recipe_id, auth=False)
-    if 'authorization_token' in recipe.args:
-        clone_url = None
-    else:
-        clone_url = util.data_url_for('data_view', recipe, cloned=True)
-    return flask.render_template('admin-recipe-view.html', recipe=recipe, clone_url=clone_url)
-
-
-# needs tests
-@app.route("/admin/recipes/<recipe_id>/edit.html")
-def admin_recipe_edit(recipe_id):
-    """ Edit a saved recipe """
-    admin.admin_auth()
-    recipe = recipes.Recipe(recipe_id, auth=False)
-    args = json.dumps(recipe.args, indent=4)
-    return flask.render_template('admin-recipe-edit.html', recipe=recipe, args=args)
-
-
-# needs tests
-@app.route("/admin/recipes/<recipe_id>/delete.html")
-def admin_recipe_delete(recipe_id):
-    """ Delete a saved recipe """
-    admin.admin_auth()
-    recipe = recipes.Recipe(recipe_id, auth=False)
-    return flask.render_template('admin-recipe-delete.html', recipe=recipe)
-
-
-# needs tests
-@app.route("/admin/recipes/")
-def admin_recipe_list():
-    """ List all saved recipes """
-    admin.admin_auth()
-    recipes = admin.admin_get_recipes()
-    return flask.render_template('admin-recipe-list.html', recipes=recipes)
-
-
-# needs tests
-@app.route("/admin/")
-def admin_root():
-    """ Root of admin pages """
-    admin.admin_auth()
-    return flask.render_template('admin-root.html')
-
-
-# needs tests
-@app.route("/admin/actions/login", methods=['POST'])
-def do_admin_login():
-    """ POST controller for an admin login """
-    password = flask.request.form.get('password')
-    admin.do_admin_login(password)
-    flask.flash("Logged in as admin")
-    return flask.redirect('/admin/', 303)
-
-
-# needs tests
-@app.route("/admin/actions/logout", methods=['POST'])
-def do_admin_logout():
-    """ POST controller for an admin logout """
-    admin.admin_auth()
-    admin.do_admin_logout()
-    flask.flash("Logged out of admin functions")
-    return flask.redirect('/data/source', 303)
-
-
-# needs tests
-@app.route("/admin/actions/update-recipe", methods=['POST'])
-def do_admin_update_recipe():
-    admin.admin_auth()
-    recipe_id = flask.request.form.get('recipe_id')
-    admin.do_admin_update_recipe(dict(flask.request.form))
-    flask.flash("Updated recipe {}".format(recipe_id))
-    return flask.redirect('/admin/recipes/{}/'.format(recipe_id), 303)
-
-
-# needs tests
-@app.route("/admin/actions/delete-recipe", methods=['POST'])
-def do_admin_delete_recipe():
-    admin.admin_auth()
-    recipe_id = flask.request.form.get('recipe_id')
-    admin.do_admin_delete_recipe(recipe_id)
-    flask.flash("Deleted recipe {}".format(recipe_id))
-    return flask.redirect('/admin/recipes/'.format(recipe_id), 303)
-
-
 
 ########################################################################
 # Controllers for extra API calls
@@ -1080,6 +996,7 @@ def do_admin_delete_recipe():
 ########################################################################
 
 @app.route("/api/from-spec.<format>")
+@util.structlogged
 def from_spec(format="json"):
     """ Use a JSON HXL spec
     Not cached
@@ -1107,14 +1024,13 @@ def from_spec(format="json"):
             'api-from-spec.html',
             spec_json=spec_json,
             spec_url=spec_url,
-            verify_ssl=util.check_verify_ssl(flask.request.args),
             filename=filename,
             force=force,
         )
     elif spec_url and spec_json:
         raise ValueError("Must specify only one of &spec-url or &spec-json")
     elif spec_url:
-        spec_response = requests.get(spec_url, verify=verify_ssl, headers=http_headers)
+        spec_response = requests.get(spec_url, headers=http_headers)
         spec_response.raise_for_status()
         spec = spec_response.json()
     elif spec_json:
@@ -1172,6 +1088,7 @@ def from_spec(format="json"):
 @app.route("/api/hxl-test")
 @app.route("/hxl-test.<format>") # legacy path
 @app.route("/hxl-test") # legacy path
+@util.structlogged
 def hxl_test(format='html'):
     """ Flask controller: test if a resource is HXL hashtagged
     GET parameters:
@@ -1232,6 +1149,7 @@ def hxl_test(format='html'):
 # has tests
 @app.route('/api/data-preview.<format>')
 #@cache.cached(key_prefix=util.make_cache_key, unless=util.skip_cache_p) # can't cache generator output
+@util.structlogged
 def data_preview (format="json"):
     """ Return a raw-data preview of any data source supported by the HXL Proxy
     Does not attempt HXL processing.
@@ -1340,6 +1258,7 @@ def data_preview (format="json"):
 # has no tests
 @app.route('/api/data-preview-sheets.<format>')
 # @cache.cached(key_prefix=util.make_cache_key, unless=util.skip_cache_p) # can't cache generator output
+@util.structlogged
 def data_preview_sheets(format="json"):
     """ Return names only for the sheets in an Excel workbook.
     In case of csv it returns one sheet name 'Default'
@@ -1407,6 +1326,7 @@ def data_preview_sheets(format="json"):
                     _output.append(str(sheet))
 
     except HXLIOException as ex:
+        logup("Found the last sheet of the Excel file", level='debug')
         logger.debug("Found the last sheet of the Excel file")
 
     # Generate result
@@ -1427,6 +1347,7 @@ def data_preview_sheets(format="json"):
 @app.route('/api/pcodes/<country>-<level>.csv')
 @app.route('/pcodes/<country>-<level>.csv') # legacy path
 @cache.cached(timeout=604800) # 1 week cache
+@util.structlogged
 def pcodes_get(country, level):
     """ Flask controller: look up a list of P-codes from iTOS
     @param country: the ISO3 country code
@@ -1449,6 +1370,7 @@ def pcodes_get(country, level):
 # has tests
 @app.route('/api/hash')
 @app.route('/hash') # legacy path
+@util.structlogged
 def make_hash():
     """ Flask controller: hash a HXL dataset
     GET parameters:
@@ -1487,6 +1409,7 @@ def make_hash():
 
 # has tests
 @app.route('/api/source-info')
+@util.structlogged
 def make_info():
     """ Flask controller: get info for an Excel dataset
     GET parameters:
@@ -1520,6 +1443,7 @@ def make_info():
 
 # needs tests
 @app.route("/")
+@util.structlogged
 def home():
     """ Flask controller: nothing currently at root
     Redirect to the /data/source page
@@ -1528,20 +1452,4 @@ def home():
     return flask.redirect(flask.url_for("data_source", **flask.request.args) , 302)
 
 
-# has tests
-@app.route('/data/<recipe_id>/chart')
-@app.route('/data/chart')
-def data_chart(recipe_id=None):
-    """ Flask controller: discontinued charting endpoint """
-    return "The HXL Proxy no longer supports basic charts. Please visit <a href='https://tools.humdata.org/'>tools.humdata.org</a>", 410
-
-
-# has tests
-@app.route('/data/<recipe_id>/map')
-@app.route('/data/map')
-def data_map(recipe_id=None):
-    """ Flask controller: discontinued mapping endpoint """
-    return "The HXL Proxy no longer supports basic maps. Please visit <a href='https://tools.humdata.org/'>tools.humdata.org</a>", 410
-
-
 # end
